@@ -1,11 +1,7 @@
-from __future__ import unicode_literals
-
 from django.db import models
 from django.db.models import ForeignKey, CASCADE
-from nested_admin.tests.compat import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class User(models.Model):
     name = models.CharField(max_length=10)
 
@@ -13,7 +9,6 @@ class User(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Project(models.Model):
     name = models.CharField(max_length=10)
     user = ForeignKey(User, on_delete=CASCADE)
@@ -22,7 +17,6 @@ class Project(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Document(models.Model):
     name = models.CharField(max_length=10)
     project = ForeignKey(Project, on_delete=CASCADE)
