@@ -7,7 +7,12 @@ from .models import TopLevel, LevelOne, LevelTwo, LevelThree
 class LevelThreeInline(NestedStackedInline):
     model = LevelThree
     extra = 1
-    inline_classes = ("collapse", "open", "grp-collapse", "grp-open",)
+    inline_classes = (
+        "collapse",
+        "open",
+        "grp-collapse",
+        "grp-open",
+    )
     sortable_field_name = "position"
 
 
@@ -15,7 +20,12 @@ class LevelTwoInline(NestedStackedInline):
     model = LevelTwo
     extra = 1
     inlines = [LevelThreeInline]
-    inline_classes = ("collapse", "open", "grp-collapse", "grp-open",)
+    inline_classes = (
+        "collapse",
+        "open",
+        "grp-collapse",
+        "grp-open",
+    )
     sortable_field_name = "position"
 
 
@@ -23,7 +33,12 @@ class LevelOneInline(NestedStackedInline):
     model = LevelOne
     extra = 1
     inlines = [LevelTwoInline]
-    inline_classes = ("collapse", "open", "grp-collapse", "grp-open",)
+    inline_classes = (
+        "collapse",
+        "open",
+        "grp-collapse",
+        "grp-open",
+    )
     sortable_field_name = "position"
 
 
